//
//  CommonViewController.m
//  DNSResolverDemo
//
//  Created by 阿里 on 2021/1/8.
//

#import "CommonViewController.h"
#import "DNSResolver.h"
@interface CommonViewController ()<NSURLSessionDelegate>
@property (nonatomic, strong) NSMutableURLRequest *mutableReq;
@property (nonatomic,strong)UITextView *resultTextView;
@end

@implementation CommonViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    self.title = @"普通场景";
    [self creatUI];
    NSString *domain = @"www.baidu.com";
    __weak typeof(self) ws = self;
//方式一
//    [[DNSResolver share] getIpsDataWithDomain:domain complete:^(NSArray<NSString *> *dataArray) {
//        NSString *ip = dataArray.firstObject;
//        if (ip.length > 0) {
//            //通过ip进行url连接
//            [ws requestWithIP:ip domain:domain];
//        }else{
//            //温馨提示：请添加兜底处理(使用域名)
//            [ws requestWithIP:domain domain:domain];
//        }
//    }];
    
//方式二
    NSArray *array = [[DNSResolver share] getIpsByCacheWithDomain:domain andExpiredIPEnabled:YES];
    NSString *ip = array.firstObject;
    if (ip.length > 0) {
        //通过ip进行url连接
        [ws requestWithIP:ip domain:domain];
    }else{
        //温馨提示：请添加兜底处理(使用域名)
        [ws requestWithIP:domain domain:domain];
    }
}

-(void)creatUI{
    self.resultTextView = [[UITextView alloc]initWithFrame:CGRectMake(0, 200, [UIScreen mainScreen].bounds.size.width, 200)];
    self.resultTextView.backgroundColor = [UIColor lightGrayColor];
    [self.view addSubview:self.resultTextView];
}

#pragma mark 普通场景
- (void)requestWithIP:(NSString *)ip domain:(NSString *)domain {
    __weak typeof(self) ws = self;
    NSURL *httpDnsURL = [NSURL URLWithString:[NSString stringWithFormat:@"https://%@", ip]];
    NSMutableURLRequest *mutableReq = [NSMutableURLRequest requestWithURL:httpDnsURL cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval: 10];
    //设置host为原始域名
    [mutableReq setValue:domain forHTTPHeaderField:@"host"];
    self.mutableReq = mutableReq;
    
    NSURLSessionConfiguration *config = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSURLSession *sessison = [NSURLSession sessionWithConfiguration:config delegate:self delegateQueue:[NSOperationQueue currentQueue]];
    
    NSURLSessionTask *task = [sessison dataTaskWithRequest:mutableReq completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        if (!error) {
            NSString *string = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
            NSLog(@"_________response: %@", string);
            ws.resultTextView.text = string;
            [sessison finishTasksAndInvalidate];
        }else{
            NSLog(@"_________error: %@", error);
        }
    }];
    
    [task resume];
}

- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task didReceiveChallenge:(NSURLAuthenticationChallenge *)challenge completionHandler:(void (^)(NSURLSessionAuthChallengeDisposition disposition, NSURLCredential * __nullable credential))completionHandler {
    if (!challenge) {
        return;
    }

    NSURLSessionAuthChallengeDisposition disposition = NSURLSessionAuthChallengePerformDefaultHandling;
    NSURLCredential *credential = nil;

    //获取原始域名信息
    NSString *host = [[self.mutableReq allHTTPHeaderFields] objectForKey:@"host"];
    if (!host) {
        host = self.mutableReq.URL.host;
    }
    if ([challenge.protectionSpace.authenticationMethod isEqualToString:NSURLAuthenticationMethodServerTrust]) {
        if ([self evaluateServerTrust:challenge.protectionSpace.serverTrust forDomain:host]) {
            disposition = NSURLSessionAuthChallengeUseCredential;
            credential = [NSURLCredential credentialForTrust:challenge.protectionSpace.serverTrust];
        } else {
            disposition = NSURLSessionAuthChallengePerformDefaultHandling;
        }
    } else {
        disposition = NSURLSessionAuthChallengePerformDefaultHandling;
    }
    completionHandler(disposition,credential);
}

- (BOOL)evaluateServerTrust:(SecTrustRef)serverTrust forDomain:(NSString *)domain {
    NSMutableArray *policies = [NSMutableArray array];
    if (domain) {
        [policies addObject:(__bridge_transfer id)SecPolicyCreateSSL(true, (__bridge CFStringRef)domain)];
    } else {
        [policies addObject:(__bridge_transfer id)SecPolicyCreateBasicX509()];
    }
    SecTrustSetPolicies(serverTrust, (__bridge CFArrayRef)policies);
    SecTrustResultType result;
    SecTrustEvaluate(serverTrust, &result);
    return (result == kSecTrustResultUnspecified || result == kSecTrustResultProceed);
}

@end
