//
// SNIViewController.m
//
// SNI应用场景
//

#import "SNIViewController.h"
#import "CFHTTPDNSHTTPProtocol.h"
#import "DNSResolver.h"

/**
 *  本示例仅用于演示HTTPS SNI场景下的处理方式。
 *  Demo为实现统一的NSURLProtocol，统一在`CFHTTPDNSProtocol`拦截请求后进行HTTPDNS域名解析。
 *  由于在SNI场景下，网络请求必须基于底层CFNetwork完成，建议不要拦截非SNI场景的网络请求，尽可能走上层网络库发送网络请求。
 *  iOS14原生方案：https://help.aliyun.com/document_detail/203723.html
 */
@interface SNIViewController () <NSURLSessionTaskDelegate, NSURLSessionDataDelegate>
@property (nonatomic,strong)UITextView *resultTextView;
@end

@implementation SNIViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    self.title = @"SNI场景";
    [self creatUI];
    [self creatData];
}

-(void)creatData{
    //注：在AppDelegate中预解析该域名
    NSString *urlString = @"https://www.douyin.com";
    NSURL *url = [NSURL URLWithString:urlString];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    /*
     *  NSURLSession加载资源场景
     */
    NSURLSessionConfiguration *configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSString *ip = [[DNSResolver share] getIpsByCacheWithDomain:@"www.douyin.com" andExpiredIPEnabled:YES].firstObject;
    if (ip){
        NSArray *protocolArray = @[ [CFHTTPDNSHTTPProtocol class] ];
        configuration.protocolClasses = protocolArray;
    }
    NSURLSession *session = [NSURLSession sessionWithConfiguration:configuration delegate:self delegateQueue:[NSOperationQueue mainQueue]];
    NSURLSessionTask *task = [session dataTaskWithRequest:request];
    [task resume];
}

-(void)creatUI{
    self.resultTextView = [[UITextView alloc]initWithFrame:CGRectMake(0, 200, [UIScreen mainScreen].bounds.size.width, 200)];
    self.resultTextView.backgroundColor = [UIColor lightGrayColor];
    [self.view addSubview:self.resultTextView];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark NSURLSessionDataDelegate
- (void)URLSession:(NSURLSession *)session dataTask:(NSURLSessionDataTask *)dataTask didReceiveData:(NSData *)data {
    NSLog(@"data: %@", [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
    self.resultTextView.text = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    [session finishTasksAndInvalidate];
}

- (void)URLSession:(NSURLSession *)session dataTask:(NSURLSessionDataTask *)dataTask didReceiveResponse:(NSURLResponse *)response completionHandler:(void (^)(NSURLSessionResponseDisposition))completionHandler {
    NSLog(@"response: %@", response);
    completionHandler(NSURLSessionResponseAllow);
}

- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task didCompleteWithError:(NSError *)error {
    if (error) {
        NSLog(@"error: %@", error);
    }
    else
        NSLog(@"complete");
}

@end
